#!/bin/bash
#
# Copyright (c) 2018 Zubax Robotics, zubax.com
# Author: Pavel Kirienko <pavel.kirienko@zubax.com>
#

function git_is_clean()
{
    [[ $(git diff --shortstat 2> /dev/null | tail -n1) == "" ]] &&\
    [[ $(git status --porcelain 2>/dev/null| grep "^??" | wc -l) == "0" ]]
}

build_environment_description=$1
if [[ "$build_environment_description" == "" ]]; then
    echo "The first argument must be the build environment description" >&2
    exit 1
fi

if [[ ${#build_environment_description} -gt 80 ]]; then
    echo "Build environment description is too long: '$build_environment_description'" >&2
    exit 1
fi

cat <<EOF
/*
 * AUTOGENERATED, DO NOT EDIT.
 * Generated on `date` by $0
 */

#pragma once

#include <cstdint>

namespace current_build_info
{

static constexpr std::uint32_t VCSCommitID = 0x`git rev-parse --short=8 HEAD`UL;

static constexpr std::uint32_t BuildTimestampUTC = `date --utc +%s`UL;

static constexpr bool BuildIsDirty = `git_is_clean && echo false || echo true`;

static const char* const BuildEnvironmentDescription =
    "$build_environment_description";

}
EOF
