/**
 * Copyright (c) 2016-2017  Zubax Robotics  <info@zubax.com>
 */

#include "board.hpp"
#include <ch.hpp>
#include <cstring>
#include <unistd.h>
#include <numeric>
#include <bootloader_interface/bootloader_interface.hpp>

namespace board
{
Watchdog init(std::chrono::milliseconds watchdog_timeout)
{
    halInit();
    chibios_rt::System::init();
    sdStart(&STDOUT_SD, nullptr);
    Watchdog wdt(watchdog_timeout);
    wdt.kick();
    return wdt;
}

__attribute__((noreturn)) void die(int reason)
{
    chibios_rt::System::halt(senoval::convertIntToString(reason));
    while (1)
    {
    }
}

void restart()
{
    NVIC_SystemReset();
}

}  // namespace board

/*
 * Early init from ChibiOS
 */
extern "C"
{
    void __early_init(void)
    {
        initGPIO();
        stm32_clock_init();

        /*
         * Making sure all peripherals that could be used by the bootloader are reset or disabled!
         * The bootloader is expected to shut everything down, but one can never be too safe.
         */
        // CAN
        RCC->APB1RSTR |= (RCC_APB1RSTR_CAN1RST | RCC_APB1RSTR_CAN2RST);
        RCC->APB1RSTR &= ~(RCC_APB1RSTR_CAN1RST | RCC_APB1RSTR_CAN2RST);

        CAN1->IER = CAN2->IER = 0;                              // Disable interrupts
        CAN1->MCR = CAN2->MCR = CAN_MCR_SLEEP | CAN_MCR_RESET;  // Software reset

        NVIC_ClearPendingIRQ(CAN1_RX0_IRQn);
        NVIC_ClearPendingIRQ(CAN1_RX1_IRQn);
        NVIC_ClearPendingIRQ(CAN1_TX_IRQn);
        NVIC_ClearPendingIRQ(CAN1_SCE_IRQn);

        NVIC_ClearPendingIRQ(CAN2_RX0_IRQn);
        NVIC_ClearPendingIRQ(CAN2_RX1_IRQn);
        NVIC_ClearPendingIRQ(CAN2_TX_IRQn);
        NVIC_ClearPendingIRQ(CAN2_SCE_IRQn);

        // USB
        RCC->AHB2RSTR |= RCC_AHB2RSTR_OTGFSRST;
        RCC->AHB2RSTR &= ~RCC_AHB2RSTR_OTGFSRST;

        NVIC_ClearPendingIRQ(OTG_FS_IRQn);
        NVIC_ClearPendingIRQ(OTG_FS_WKUP_IRQn);
    }

    void boardInit(void) {}
}
